<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;

/**
 * Reporter for VirtueMart helper.
 */
abstract class VMReporterHelper
{
	protected static $columns = array();
	
	protected static $fields = null;
	
	protected static $userfields = null;
	
	protected static $site_template = null;
	
	protected static $extensions;
	/**
	 * Configure the Linkbar.
	 */
	public static function addSubmenu($vName)
	{
		JHtmlSidebar::addEntry(
			JText::_('COM_VMREPORTER_MENU_TITLE_OVERVIEW'),
			'index.php?option=com_vmreporter&view=frontpage',
			$vName == 'frontpage'
		);
		
		JHtmlSidebar::addEntry(
			JText::_('COM_VMREPORTER_MENU_TITLE_REPORTS'),
			'index.php?option=com_vmreporter&view=reports',
			$vName == 'reports'
		);
		
		JHtmlSidebar::addEntry(
			JText::_('COM_VMREPORTER_MENU_TITLE_TEMPLATES'),
			'index.php?option=com_vmreporter&view=templates',
			$vName == 'templates'
		);
		
		JHtmlSidebar::addEntry(
			JText::_('COM_VMREPORTER_MENU_TITLE_CRONS'),
			'index.php?option=com_vmreporter&view=crons',
			$vName == 'crons'
		);
		
		JHtmlSidebar::addEntry('<hr />', '', false);
		
		JHtmlSidebar::addEntry(
			JText::_('COM_VMREPORTER_MENU_TITLE_INVENTORY'),
			'index.php?option=com_vmreporter&view=inventory',
			$vName == 'inventory'
		);
	}

	/**
	 * Gets a list of the actions that can be performed.
	 *
	 * @return  JObject
	 */
	public static function getActions()
	{
		$user   = JFactory::getUser();
		$result = new JObject;

		$assetName = 'com_vmreporter';

		$actions = array(
			'core.admin', 'core.manage', 'core.create', 'core.edit', 'core.edit.own', 'core.edit.state', 'core.delete'
		);

		foreach ($actions as $action) {
			$result->set($action, $user->authorise($action, $assetName));
		}

		return $result;
	}
	
	public static function getOrderUserinfoColumns($type, $ignore = array(), $selectedOnly = true)
	{
		$type = strtoupper($type);
		
		if (!in_array($type, array('BT', 'ST')))
		{
			return array();
		}
		
		if (empty($ignore))
		{
			$ignore = array('created_on', 'created_by', 'address_type', 'virtuemart_order_id', 'modified_on', 'modified_by', 'locked_on', 'locked_by');
		}
		
		$columns = self::getColumns('#__virtuemart_order_userinfos', $ignore, $selectedOnly);
		
		if (empty($columns))
		{
			return array();
		}
		
		$userfields = self::getUserfields($type, false);
		
		return array_intersect($columns, $userfields);
	}
	
	public static function getUserfields($type, $withTitle = false)
	{
		$type = strtoupper($type);
		
		if (!in_array($type, array('ALL', 'BT', 'ST')))
		{
			$type = 'ALL';
		}
		
		if (self::$userfields === null)
		{
			$db    = JFactory::getDbo();
			$query = $db->getQuery(true);
			
			$query->select('*')
	      ->from($db->quoteName('#__virtuemart_userfields'))
	      ->where($db->quoteName('published') . ' = 1');

			$db->setQuery($query);
			self::$userfields = $db->loadObjectList();
		}
		
		$fields = array();
		
		foreach (self::$userfields as $userfield)
		{
			if ($type == 'BT' && (!empty($userfield->registration) || !empty($userfield->account) || !empty($userfield->cart)))
			{
				$fields[$userfield->name] = $userfield;
			}
			elseif ($type == 'ST' && !empty($userfield->shipment))
			{
				$fields[$userfield->name] = $userfield;
			}
			elseif ($type == 'ALL')
			{
				$fields[$userfield->name] = $userfield;
			}
		}
		
		if (!empty($fields) && !$withTitle)
		{
			return array_keys($fields);
		}
		
		return $fields;
	}
	
	public static function getColumns($table, $ignore = array(), $selectedOnly = false)
	{
		$db = JFactory::getDbo();
		
		// Lookup the columns for this table only once.
		if (!array_key_exists($table, self::$columns))
		{
			$columns = $db->getTableColumns($table, false);
			
			if (empty($columns))
			{
				$e = new JException(JText::_('JLIB_DATABASE_ERROR_COLUMNS_NOT_FOUND'));
				$this->setError($e);
				return false;
			}
			
			self::$columns[$table] = array_keys($columns);
		}
		
		$columns = array_diff(self::$columns[$table], $ignore);
		
		if ($selectedOnly)
		{
			if (self::$fields === null)
			{
				$query = $db->getQuery(true);
				$query->clear()
					->select($db->quoteName('fields'))
					->from($db->quoteName('#__vmreporter_fields'))
					->where($db->quoteName('id') . ' = 1');
				$db->setQuery($query);
				$fields = $db->loadResult();
				
				$registry = new Registry($fields);
				self::$fields = $registry->toArray();
			}
			
			if (!empty(self::$fields) && is_array(self::$fields))
			{
				$columns = array_intersect($columns, self::$fields);
			}
		}

		return $columns;
	}
	
	public static function getFieldLabel($field)
	{
		$userfields = self::getUserfields('ALL', true);
		
		if (!empty($userfields) && isset($userfields[$field]) && is_object($userfields[$field]))
		{
			return $userfields[$field]->title;
		}
		
		return strtoupper('COM_VMREPORTER_LBL_' . $field);
	}
	
	public static function getCurrencyFormat()
	{
		static $currency = null;
		
		if ($currency === null)
		{
			$currency_id = CurrencyDisplay::getInstance()->getId();
			
			$db = JFactory::getDbo();
			$query = $db->getQuery(true);
			
			$query->select($db->quoteName(array('currency_symbol', 'currency_decimal_place', 'currency_decimal_symbol', 'currency_thousands', 'currency_positive_style')))
				->from($db->quoteName('#__virtuemart_currencies'))
				->where($db->quoteName('virtuemart_currency_id') . ' = ' . (int) $currency_id);

			$db->setQuery($query);
			$currency = $db->loadObject();
			
			if (empty($currency))
			{
				$currency = new stdClass;
				$currency->currency_symbol = '';
				$currency->currency_decimal_place = 2;
				$currency->currency_decimal_symbol = '.';
				$currency->currency_thousands = ',';
				$currency->currency_positive_style = '{number} {symbol}';
			}
		}
		
		return $currency;
	}
	
	public static function formatPrice($price, $number_format)
	{
		static $currency = null;
		
		if ($currency === null)
		{
			$currency = CurrencyDisplay::getInstance();
		}

		$price = $currency->roundForDisplay($price);
		
		if ($number_format)
		{
			$currency_format = self::getCurrencyFormat();
			
			if ($currency_format)
			{
				$price = number_format((float) $price, (int) $currency_format->currency_decimal_place);
			}
		}
		
		return $price;
	}
	
	public static function roundPrices($row, $number_format = false)
	{
		static $currency = null;
		
		$price_fields_order = array(
			'order_total', 'order_salesPrice', 'order_billTaxAmount',
			'order_billDiscountAmount', 'order_discountAmount', 'order_subtotal',
			'order_tax', 'order_shipment', 'order_shipment_tax',
			'order_payment', 'order_payment_tax', 'coupon_discount',
			'order_discount'
		);
		
		$price_fields_order_items = array(
			'product_item_price', 'product_priceWithoutTax', 'product_tax',
			'product_basePriceWithTax', 'product_discountedPriceWithoutTax', 'product_final_price',
			'product_subtotal_discount', 'product_subtotal_with_tax'
		);
		
		$price_fields = array_merge($price_fields_order, $price_fields_order_items);
		
		foreach ($price_fields as $price_field)
		{
			if (isset($row[$price_field]))
			{
				if (empty($row[$price_field]))
				{
					$row[$price_field] = 0;
				}
				
				$row[$price_field] = self::formatPrice($row[$price_field], $number_format);
			}
		}

		return $row;
	}
	
	public static function renderLayout($layoutFile, $displayData = null)
	{
		$basePath = null;
		$options  = new Registry(array('client' => 'site', 'component' => 'com_vmreporter'));
		$layout   = new JLayoutFile($layoutFile, $basePath, $options);
		
		if ($template = self::getTemplate())
		{
			$layout->addIncludePath(JPATH_SITE . '/templates/' . $template . '/html/layouts/com_vmreporter');
		}
		
		return $layout->render($displayData);
	}
	
	public static function getTemplate()
	{
		if (self::$site_template === null)
		{
			$db    = JFactory::getDbo();
			$query = $db->getQuery(true);
			
			$query->select('template')
				->from('#__template_styles as s')
				->where('s.client_id = 0')
				->where('s.home = 1')
				->where('e.enabled = 1')
				->join('LEFT', '#__extensions as e ON e.element=s.template AND e.type=' . $db->quote('template') . ' AND e.client_id=s.client_id');
				
			$db->setQuery($query);
			self::$site_template = $db->loadResult();
		}
		
		return self::$site_template;
	}
	
	public static function checkSystem()
	{
		$app    = JFactory::getApplication();
		$config = JFactory::getConfig();
		
		// If error reporting is enabled and PHP version is greater than or equal to 8.1.0.
		if ((($config->get('error_reporting', 'none') === 'maximum' || $config->get('error_reporting', 'none') === 'development') || (VmConfig::get('vmdev', 'none') === 'admin' || VmConfig::get('vmdev', 'none') === 'all')) && version_compare(PHP_VERSION, '8.1.0', 'ge') && function_exists('error_reporting'))
		{
			// Disable deprecation warnings.
			error_reporting(~E_DEPRECATED);
		}
		
		// If the system plugin is not installed or enabled.
		if (!self::hasExtension('vmreporter', 'plugin', 'system'))
		{
			$app->enqueueMessage(JText::_('COM_VMREPORTER_ERROR_SYSTEM_PLUGIN_DISABLED'), 'error');
		}
		
		if (self::hasExtension('pkg_vmreporter', 'package'))
		{
			$params      = JComponentHelper::getParams('com_vmreporter');
			$downloadKey = $params->get('download_key', '');
			$downloadKey = $downloadKey ? trim($downloadKey) : null;
			
			if (empty($downloadKey))
			{
				$app->enqueueMessage(JText::_('COM_VMREPORTER_WARNING_NO_DOWNLOAD_KEY'), 'info');
			}
		}
	}

	public static function getDefaultShopperGroup($guest = false)
	{
		static $group = array();

		if (!isset($group[$guest]))
		{
			$db    = JFactory::getDbo();
			$query = $db->getQuery(true);

			$query->select('*')
				->from($db->quoteName('#__virtuemart_shoppergroups'));

			if ($guest)
			{
				$query->where($db->quoteName('default') . ' = 2');
			}
			else
			{
				$query->where($db->quoteName('default') . ' = 1');
			}

			$db->setQuery($query);
			$group[$guest] = $db->loadObject();

			if (!empty($group[$guest]))
			{
				$group[$guest]->virtuemart_shoppergroup_id = (int) $group[$guest]->virtuemart_shoppergroup_id;
			}
		}

		return $group[$guest];
	}

	public static function getUsersByShopperGroups($groups)
	{
		static $users = array();

		$groups = array_map('intval', $groups);
		$hash   = md5(serialize($groups));

		if (empty($groups))
		{
			return array();
		}

		if (!isset($users[$hash]))
		{
			$db    = JFactory::getDbo();
			$query = $db->getQuery(true);

			$query->select($db->quoteName('virtuemart_user_id'))
				->from($db->quoteName('#__virtuemart_vmuser_shoppergroups'));

			if (count($groups) == 1)
			{
				$query->where($db->quoteName('virtuemart_shoppergroup_id') . ' = ' . $groups[0]);
			}
			else
			{
				$query->where($db->quoteName('virtuemart_shoppergroup_id') . ' IN (' . implode(',', $groups) . ')');
			}

			$db->setQuery($query);
			$users[$hash] = $db->loadColumn();

			if (empty($users[$hash]))
			{
				$users[$hash] = array();
			}
			else
			{
				$users[$hash] = array_map('intval', $users[$hash]);
			}
		}

		return $users[$hash];
	}

	public static function getOrderIdsByCalculationRules($rules)
	{
		static $orders = array();

		$rules = array_map('intval', $rules);
		$hash   = md5(serialize($rules));

		if (empty($rules))
		{
			return array();
		}

		if (!isset($orders[$hash]))
		{
			$db    = JFactory::getDbo();
			$query = $db->getQuery(true);

			$query->select('DISTINCT ' . $db->quoteName('virtuemart_order_id'))
			  ->from($db->quoteName('#__virtuemart_order_calc_rules'));

			if (count($rules) == 1)
			{
				$query->where($db->quoteName('virtuemart_calc_id') . ' = ' . $rules[0]);
			}
			else
			{
				$query->where($db->quoteName('virtuemart_calc_id') . ' IN (' . explode(',', $rules) . ')');
			}

			$db->setQuery($query);
			$orders[$hash] = $db->loadColumn();

			if (empty($orders[$hash]))
			{
				$orders[$hash] = array();
			}
			else
			{
				$orders[$hash] = array_map('intval', $orders[$hash]);
			}
		}

		return $orders[$hash];
	}
	
	/**
	* Method to check if an extension installed and enabled
	* 
	* @param  string  $element  Name of the extension
	* @param  string  $type     Type of the extension like component, module, plugins etc
	* 
	* @return boolean
	*/
	public static function hasExtension($element, $type = 'component', $folder = null)
	{
		if (!is_array(self::$extensions))
		{
			self::$extensions = array();
		}
		
		$hash = 'element:' . $element . '.type:' . $type;
		
		if (!empty($folder))
		{
			$hash .= '.folder:' . $folder;
		}
		
		if (!array_key_exists($hash, self::$extensions))
		{
			$type = strtolower($type);
			self::$extensions[$hash] = false;
			
			$db    = JFactory::getDbo();
			$query = $db->getQuery(true);

			$query->select($db->quoteName(array('extension_id', 'element', 'folder')))
				->from($db->quoteName('#__extensions'))
				->where($db->quoteName('type') . ' = ' . $db->quote($type))
				->where($db->quoteName('element') . ' = ' . $db->quote($element))
				->where($db->quoteName('enabled') . ' = 1');
			
			try
			{
				$db->setQuery($query);
				$results = $db->loadObjectList();
			}
			catch(Exception $e)
			{
				JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
				return false;
			}
			
			if (!empty($results))
			{
				foreach ($results as $result)
				{
					if ($type != 'package')
					{
						switch($type)
						{
							case 'module':
								$path = JPATH_SITE . '/modules/' . $result->element;
								break;
							case 'plugin':
								if (!empty($folder) && strtolower($folder) != $result->folder)
								{
									$path = null;
								}
								else
								{
									$path = JPATH_SITE . '/plugins/' . $result->folder . '/' . $result->element;
								}
								break;
							default:
							case 'component':
								$path = JPATH_ADMINISTRATOR . '/components/' . $result->element;
						}
						
						if ($path && file_exists($path) && is_dir($path))
						{
							self::$extensions[$hash] = true;
							break;
						}
					}
					else
					{
						self::$extensions[$hash] = true;
						break;
					}
				}
			}
		}

		return self::$extensions[$hash];
	}
}
