<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;

/**
 * VMReporter cron model.
 */
class VMReporterModelCron extends JModelAdmin
{
	/**
	 * @var      string    The prefix to use with controller messages.
	 */
	protected $text_prefix = 'COM_VMREPORTER';

	/**
	 * @var      string    Alias to manage history control
	 */
	public $typeAlias = 'com_vmreporter.cron';

	/**
	 * @var null  Item data
	 */
	protected $item = null;
	
	
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JModelLegacy
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);
	}
	
	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param   string  $type    The table type to instantiate
	 * @param   string  $prefix  A prefix for the table class name. Optional.
	 * @param   array   $config  Configuration array for model. Optional.
	 *
	 * @return    JTable    A database object
	 */
	public function getTable($type = 'Cron', $prefix = 'VMReporterTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	/**
	 * Method to get the record form.
	 *
	 * @param   array    $data      An optional array of data for the form to interogate.
	 * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
	 *
	 * @return  JForm  A JForm object on success, false on failure
	 */
	public function getForm($data = array(), $loadData = true)
	{
		// Initialise variables.
		$app = JFactory::getApplication();

		// Get the form.
		$form = $this->loadForm('com_vmreporter.cron', 'cron', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}

		return $form;
	}

	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return   mixed  The data for the form.
	 */
	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_vmreporter.edit.cron.data', array());

		if (empty($data))
		{
			if ($this->item === null)
			{
				$this->item = $this->getItem();
			}

			$data = $this->item;
		}

		return $data;
	}

	/**
	 * Method to get a single record.
	 *
	 * @param   integer  $pk  The id of the primary key.
	 *
	 * @return  mixed    Object on success, false on failure.
	 */
	public function getItem($pk = null)
	{
		if ($item = parent::getItem($pk))
		{
			if (!empty($item->log) && is_string($item->log))
			{
				$registry = new Registry($item->log);
				$item->log = $registry->toArray();
			}
			
			if (!empty($item->time_interval))
			{
				if ($item->time_interval_uom == 'minutes' && $item->time_interval <= (59 * 60))
				{
					$item->minutes = $item->time_interval / 60;
				}
				
				if ($item->time_interval_uom == 'hours' && $item->time_interval > (59 * 60) && $item->time_interval <= (23 * 3600))
				{
					$item->hours = $item->time_interval / 3600;
				}
				
				if ($item->time_interval_uom == 'days' && $item->time_interval > (23 * 3600) && $item->time_interval <= (6 * 86400))
				{
					$item->days = $item->time_interval / 86400;
				}
				
				if ($item->time_interval_uom == 'weeks' && $item->time_interval > (6 * 86400))
				{
					$item->weeks = $item->time_interval / 604800;
				}
				
				if ($item->time_interval_uom == 'months' && $item->time_interval > (29 * 86400))
				{
					$item->months = $item->time_interval / (30 * 86400);
				}
			}
		}

		return $item;
	}

	/**
	 * Method to save the form data.
	 *
	 * @param   array  $data  The form data.
	 *
	 * @return  boolean  True on success, False on error.
	 */
	public function save($data)
	{
		if (empty($data['time_interval_uom']))
		{
			$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_TIME_INTERVAL_UOM'));
			return false;
		}

		switch ($data['time_interval_uom'])
		{
			case 'minutes':
				if (empty($data['minutes']))
				{
					$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_CRON_MINUTES'));
					return false;
				}
				$data['time_interval'] = $data['minutes'] * 60;
				break;
			
			case 'hours':
				if (empty($data['hours']))
				{
					$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_CRON_HOURS'));
					return false;
				}
				$data['time_interval'] = $data['hours'] * 3600;
				break;
				
			case 'days':
				if (empty($data['days']))
				{
					$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_CRON_DAYS'));
					return false;
				}
				$data['time_interval'] = $data['days'] * 86400;
				break;
				
			case 'weeks':
				if (empty($data['weeks']))
				{
					$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_CRON_WEEKS'));
					return false;
				}
				$data['time_interval'] = $data['weeks'] * 604800;
				break;
				
			case 'months':
				if (empty($data['months']))
				{
					$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_CRON_MONTHS'));
					return false;
				}
				$data['time_interval'] = $data['months'] * 30 * 86400;
				break;
			
			default:
				$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_TIME_INTERVAL_UOM'));
				return false;
				break;
		}

		return parent::save($data);
	}

	/**
	 * Method to duplicate an Report
	 *
	 * @param   array  &$pks  An array of primary key IDs.
	 *
	 * @return  boolean  True if successful.
	 *
	 * @throws  Exception
	 */
	public function duplicate(&$pks)
	{
		$user = JFactory::getUser();

		// Access checks.
		if (!$user->authorise('core.create', 'com_vmreporter'))
		{
			throw new Exception(JText::_('JERROR_CORE_CREATE_NOT_PERMITTED'));
		}

		$dispatcher = JEventDispatcher::getInstance();
		$context    = $this->option . '.' . $this->name;

		// Include the plugins for the save events.
		JPluginHelper::importPlugin($this->events_map['save']);

		$table = $this->getTable();

		foreach ($pks as $pk)
		{
			if ($table->load($pk, true))
			{
				// Reset the id to create a new record.
				$table->id = 0;
				
				// Unpublish the new record
				$table->state = 0;

				if (!$table->check())
				{
					throw new Exception($table->getError());
				}
				
				// Trigger the before save event.
				$result = $dispatcher->trigger($this->event_before_save, array($context, &$table, true));

				if (in_array(false, $result, true) || !$table->store())
				{
					throw new Exception($table->getError());
				}

				// Trigger the after save event.
				$dispatcher->trigger($this->event_after_save, array($context, &$table, true));
			}
			else
			{
				throw new Exception($table->getError());
			}
		}

		// Clean cache
		$this->cleanCache();

		return true;
	}
}
