<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;

JLoader::register('VMReporterModelReport', __DIR__ . '/report.php');

class VMReporterModelFrontpage extends VMReporterModelReport
{
	public function __construct($config = array())
	{
		parent::__construct($config);
		
		// Guess the context as Option.ModelName.
		if (empty($this->context))
		{
			$this->context = strtolower($this->option . '.' . $this->getName());
		}
	}

	/**
	* Method to auto-populate the model state.
	*
	* Note. Calling getState in this method will result in recursion.
	*/
	protected function populateState($ordering = null, $direction = null) 
	{
		// Initialise variables.
		$app    = JFactory::getApplication();
		$params = JComponentHelper::getParams('com_vmreporter');

		// Load the period.
		$period = $app->getUserStateFromRequest('com_vmreporter.frontpage.filter.period', 'filter_period', 'this_month');
		$this->setState('report.period', $period);
		
		$status = $app->getUserStateFromRequest('com_vmreporter.frontpage.filter.status', 'filter_status', array('C', 'U'), 'ARRAY');
		$this->setState('report.status', $status);
		
		$start_date = $app->getUserStateFromRequest('com_vmreporter.frontpage.filter.start_date', 'filter_start_date', '2015-01-01 00:00:00', 'STRING');
		$this->setState('report.start_date', $start_date);
		
		$end_date = $app->getUserStateFromRequest('com_vmreporter.frontpage.filter.end_date', 'filter_end_date', '2016-01-01 00:00:00', 'STRING');
		$this->setState('report.end_date', $end_date);

		// Load the parameters.
		$this->setState('params', $params);

		// List state information.
		parent::populateState('virtuemart_order_item_id', 'asc');
	}
	
	/**
	 * Method to get a store id based on the model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  An identifier string to generate the store id.
	 *
	 * @return  string  A store id.
	 */
	protected function getStoreId($id = '')
	{
		$id .= $this->getState('report.period');
		
		$status = $this->getState('report.status');
		
		$id .= serialize($status);
		$id .= $this->getState('report.start_date');
		$id .= $this->getState('report.end_date');
		
		return md5($this->context . ':' . $id);
	}
	
	public function getReport($pkey = null)
	{
		$cache_key = $this->getStoreId('generateReport');
		
		if (isset($this->report[$cache_key]))
		{
			return $this->report[$cache_key];
		}
		
		$template = $this->getReportTemplate();
		
		// Build the report from VirtueMart database tables
		$items = $this->buildNewReport($template);

		if ($items === false)
		{
			return false;
		}
		
		foreach ($items as &$item)
		{
			$this->cleanRow($item);
		}
		
		$report = new JObject;
		
		$report->id    = -1;
		$report->valid = true;
		$report->data  = $items;

		$this->report[$cache_key] = $report;
		
		return $this->report[$cache_key];
	}
	
	public function getCustomerData()
	{
		$this->setState('analysis.type', 'customer_number');
		
		return $this->getAnalysis();
	}
	
	public function getProductData()
	{
		$this->setState('analysis.type', 'virtuemart_product_id');
		
		return $this->getAnalysis();
	}
	
	public function getCategoryData()
	{
		$this->setState('analysis.type', 'category_id');
		
		return $this->getAnalysis();
	}
	
	public function getReportTemplate()
	{
		// Get a storage key.
		$store = $this->getStoreId('template');
		
		// Try to load the data from internal storage.
		if (isset($this->cache[$store]))
		{
			return $this->cache[$store];
		}
		
		$template = $this->getTable('Template');
		
		$jsonProperties = array(
			'products', 'categories',
			'manufacturers', 'users',
			'shopper_groups', 'countries',
			'st_countries', 'statuses',
			'order_fields', 'order_item_fields',
			'bt_fields', 'st_fields'
		);
		
		foreach ($jsonProperties as $property)
		{
			if (property_exists($template, $property))
			{
				$registry = new Registry($template->$property);
				$template->$property = $registry->toArray();
			}
		}
		
		$template->period     = $this->getState('report.period');
		$template->statuses   = $this->getState('report.status');
		$template->start_date = $this->getState('report.start_date');
		$template->end_date   = $this->getState('report.end_date');

		$this->cache[$store] = $template;
		
		return $this->cache[$store];
	}
	
	public function getSortFields($template = null, $items = null)
	{
		$items = ($items === null) ? $this->getItems() : $items;
		
		if (!isset($items[0]))
		{
			return array();
		}
		
		$fields = array_keys($items[0]);
		$sortableFields = array();
		
		foreach ($fields as $field)
		{
			if (strpos($field, 'shipto_') === 0)
			{
				$label = VMReporterHelper::getFieldLabel(substr($field, 7));
				
				$sortableFields[$field] = JText::_('COM_VMREPORTER_SHIPTO_PREFIX') . JText::_($label);
			}
			else
			{
				$label = VMReporterHelper::getFieldLabel($field);
				
				$sortableFields[$field] = JText::_($label);
			}
		}
		
		$template_id = 0;

		JPluginHelper::importPlugin('vmreporter');
		
		if (version_compare(JVERSION, '4.0.0', 'ge'))
		{
			JFactory::getApplication()->triggerEvent('onPrepareSortableFields', array(&$sortableFields, $template_id));
		}
		else
		{
			JDispatcher::getInstance()->trigger('onPrepareSortableFields', array(&$sortableFields, $template_id));
		}

		return $sortableFields;
	}
}
