<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

jimport('joomla.filesystem.file');

/**
 * Reporter for VirtueMart
 */
class VMReporterModelInventory extends JModelList
{
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'virtuemart_product_id', 'p.virtuemart_product_id',
				'product_name', 'parent_product_name',
				'product_sku', 'p.product_sku',
				'product_in_stock', 'p.product_in_stock',
				'product_ordered', 'p.product_ordered',
				'available_stock', 
				'product_special', 'p.product_special',
				'published', 'p.published',
				'categories', 'manufacturers'
			);
		}
		
		parent::__construct($config);
	}

	/**
	* Method to auto-populate the model state.
	*
	* Note. Calling getState in this method will result in recursion.
	*/
	protected function populateState($ordering = null, $direction = null) 
	{
		// Initialise variables.
		$app = JFactory::getApplication();

		// Load the filter state.
		$search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
		$this->setState('filter.search', $search);
		
		$available_stock = $app->getUserStateFromRequest($this->context . '.filter.available_stock', 'filter_available_stock');
		$this->setState('filter.available_stock', $available_stock);
		
		$published = $app->getUserStateFromRequest($this->context . '.filter.published', 'filter_published');
		$this->setState('filter.published', $published);
		
		$product_special = $app->getUserStateFromRequest($this->context . '.filter.product_special', 'filter_product_special');
		$this->setState('filter.product_special', $product_special);
		
		$categories = $app->input->get('filter_categories', array(), 'ARRAY');
		$categories = array_map('intval', $categories);
		$this->setState('filter.categories', $categories);
		
		$manufacturers = $app->input->get('filter_manufacturers', array(), 'ARRAY');
		$manufacturers = array_map('intval', $manufacturers);
		$this->setState('filter.manufacturers', $manufacturers);
		
		$parent_product_id = $app->getUserStateFromRequest($this->context . '.filter.parent_product_id', 'parent_product_id');
		$this->setState('filter.parent_product_id', $parent_product_id);
		
		// Load the parameters.
		$params = JComponentHelper::getParams('com_vmreporter');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('p.pordering', 'desc');
	}


	/**
	* Method to get a store id based on model configuration state.
	*
	* This is necessary because the model is used by the component and
	* different modules that might need different sets of data or different
	* ordering requirements.
	*
	* @param  string   $id A prefix for the store id.
	* @return string       A store id.
	* @since  1.0.0
	*/
	protected function getStoreId($id = '') 
	{
		// Compile the store id.
		$id.= ':' . $this->getState('filter.search');
		$id.= ':' . $this->getState('filter.available_stock');
		$id.= ':' . $this->getState('filter.published');
		$id.= ':' . $this->getState('filter.product_special');
		$id.= ':' . json_encode($this->getState('filter.categories'));
		$id.= ':' . json_encode($this->getState('filter.manufacturers'));
		$id.= ':' . $this->getState('filter.parent_product_id');

		return parent::getStoreId($id);
	}

	/**
	* Build an SQL query to load the list data.
	*
	* @return	JDatabaseQuery
	* @since	1.0.0
	*/
	protected function getListQuery() 
	{
		// Create a new query object.
		$db     = $this->getDbo();
		$query  = $db->getQuery(true);
		$params = $this->getState('params');

		// Select the required fields from the table.
		$query->select($db->quoteName(['p.virtuemart_product_id', 'p.product_parent_id']));
		
		$query->from($db->quoteName('#__virtuemart_products') . ' AS p');
		
		$query->join('LEFT', $db->quoteName('#__virtuemart_products_' . VmConfig::$vmlang) . ' AS pl ON ' . $db->quoteName('pl.virtuemart_product_id') . ' = ' . $db->quoteName('p.virtuemart_product_id'));
		
		if (VmConfig::$defaultLang != VmConfig::$vmlang && VmConfig::$langCount > 1)
		{
			$query->select('CASE WHEN ' . $db->quoteName('pl.product_name') . ' IS NULL THEN ' . $db->quoteName('pdl.product_name') . ' ELSE ' . $db->quoteName('pl.product_name') . ' END AS product_name');
			
			$query->join('LEFT', $db->quoteName('#__virtuemart_products_' . VmConfig::$defaultLang) . ' AS pdl ON ' . $db->quoteName('pdl.virtuemart_product_id') . ' = ' . $db->quoteName('p.virtuemart_product_id'));
		}
		else
		{
			$query->select($db->quoteName('pl.product_name'));
		}
		
		// Join for parent product name
		$query->join('LEFT', $db->quoteName('#__virtuemart_products_' . VmConfig::$vmlang) . ' AS ppl ON ' . $db->quoteName('p.product_parent_id') . ' > 0 AND ' . $db->quoteName('ppl.virtuemart_product_id') . ' = ' . $db->quoteName('p.product_parent_id'));
		
		if (VmConfig::$defaultLang != VmConfig::$vmlang && VmConfig::$langCount > 1)
		{
			$query->select('CASE WHEN ' . $db->quoteName('ppl.product_name') . ' IS NULL THEN ' . $db->quoteName('ppdl.product_name') . ' ELSE ' . $db->quoteName('ppl.product_name') . ' END AS parent_product_name');
			
			$query->join('LEFT', $db->quoteName('#__virtuemart_products_' . VmConfig::$defaultLang) . ' AS ppdl ON ' . $db->quoteName('p.product_parent_id') . ' > 0 AND ' . $db->quoteName('ppdl.virtuemart_product_id') . ' = ' . $db->quoteName('p.product_parent_id'));
		}
		else
		{
			$query->select($db->quoteName('ppl.product_name') . ' AS parent_product_name');
		}
		
		$query->select($db->quoteName('p.product_sku'));
		
		// If has shared stock with parent products
		if ($params->get('has_shared_inventory', 0))
		{
			$query->join('LEFT', $db->quoteName('#__virtuemart_products') . ' AS pp ON ' . $db->quoteName('p.product_parent_id') . ' > 0 AND ' . $db->quoteName('pp.virtuemart_product_id') . ' = ' . $db->quoteName('p.product_parent_id'));
			
			$query->select('CASE WHEN ' . $db->quoteName('p.product_parent_id') . ' > 0 AND ' . $db->quoteName('p.product_params') . ' LIKE ' . $db->quote('%' . $db->escape('shared_stock="1"') . '%') . ' THEN ' . $db->quoteName('pp.product_in_stock') . ' ELSE ' . $db->quoteName('p.product_in_stock') . ' END AS product_in_stock');
			
			$query->select('CASE WHEN ' . $db->quoteName('p.product_parent_id') . ' > 0 AND ' . $db->quoteName('p.product_params') . ' LIKE ' . $db->quote('%' . $db->escape('shared_stock="1"') . '%') . ' THEN ' . $db->quoteName('pp.product_ordered') . ' ELSE ' . $db->quoteName('p.product_ordered') . ' END AS product_ordered');
			
			$query->select('CASE WHEN ' . $db->quoteName('p.product_parent_id') . ' > 0 AND ' . $db->quoteName('p.product_params') . ' LIKE ' . $db->quote('%' . $db->escape('shared_stock="1"') . '%') . ' THEN (' . $db->quoteName('pp.product_in_stock') . ' - ' . $db->quoteName('pp.product_ordered') . ') ELSE (' . $db->quoteName('p.product_in_stock') . ' - ' . $db->quoteName('p.product_ordered') . ') END AS available_stock');
		}
		else
		{
			$query->select($db->quoteName(['p.product_in_stock', 'p.product_ordered']));
			
			$query->select('(' . $db->quoteName('p.product_in_stock') . ' - ' . $db->quoteName('p.product_ordered') . ') AS available_stock');
		}
		
		$query->select($db->quoteName(['p.published', 'p.product_special']));
		
		// Filter by search in title
		$search = $this->getState('filter.search');

		// Filter the items over the search string if set.
		if (!empty($search))
		{
			// Escape the search token.
			$token = $db->quote('%' . $db->escape($search) . '%');

			// Compile the different search clauses.
			$searches = [];
			
			$searches[] = $db->quoteName('p.product_sku') . ' LIKE ' . $token;
			$searches[] = $db->quoteName('pl.product_name') . ' LIKE ' . $token;
			$searches[] = $db->quoteName('ppl.product_name') . ' LIKE ' . $token;
			
			if (VmConfig::$defaultLang != VmConfig::$vmlang && VmConfig::$langCount > 1)
			{
				$searches[] = $db->quoteName('pdl.product_name') . ' LIKE ' . $token;
				$searches[] = $db->quoteName('ppdl.product_name') . ' LIKE ' . $token;
			}
			

			// Add the clauses to the query.
			$query->where('(' . implode(' OR ', $searches) . ')');
		}
		
		// Filter by availablity
		$available_stock = $this->getState('filter.available_stock');
		
		if (is_numeric($available_stock))
		{
			$condition = !empty($available_stock) ? ' > 0' : ' <= 0';
			
			if ($params->get('has_shared_inventory', 0))
			{
				$query->where('(CASE WHEN ' . $db->quoteName('p.product_parent_id') . ' > 0 AND ' . $db->quoteName('p.product_params') . ' LIKE ' . $db->quote('%' . $db->escape('shared_stock="1"') . '%') . ' THEN (' . $db->quoteName('pp.product_in_stock') . ' - ' . $db->quoteName('pp.product_ordered') . ')' . $condition . ' ELSE (' . $db->quoteName('p.product_in_stock') . ' - ' . $db->quoteName('p.product_ordered') . ')' . $condition . ' END)');
			}
			else
			{
				$query->where('(' . $db->quoteName('p.product_in_stock') . ' - ' . $db->quoteName('p.product_ordered') . ')' . $condition);
			}
		}
		
		// Filter by state
		$published = $this->getState('filter.published');
		
		if (is_numeric($published))
		{
			$query->where($db->quoteName('p.published') . ' = ' . (int) $published);
		}
		
		// Filter by featured products
		$product_special = $this->getState('filter.product_special');
		
		if (is_numeric($product_special))
		{
			$query->where($db->quoteName('p.product_special') . ' = ' . (int) $product_special);
		}
		
		// Filter by categories
		$categories = $this->getState('filter.categories');
		
		if (!empty($categories) && is_array($categories))
		{
			$categories = array_filter(array_map('intval', $categories));
			
			if (!empty($categories))
			{
				$query->join('LEFT', $db->quoteName('#__virtuemart_product_categories') . ' AS cp ON ' . $db->quoteName('cp.virtuemart_product_id') . ' = ' . $db->quoteName('p.virtuemart_product_id'));
				
				if (count($categories) == 1)
				{
					$query->where($db->quoteName('cp.virtuemart_category_id') . ' = ' . (int) $categories[0]);
				}
				else
				{
					$query->where($db->quoteName('cp.virtuemart_category_id') . ' IN (' . implode(',', $categories) . ')');
				}
			}
		}
		
		// Filter by manufactuers
		$manufacturers = $this->getState('filter.manufacturers');
		
		if (!empty($manufacturers) && is_array($manufacturers))
		{
			$manufacturers = array_filter(array_map('intval', $manufacturers));
			
			if (!empty($manufacturers))
			{
				$query->join('LEFT', $db->quoteName('#__virtuemart_product_manufacturers') . ' AS mp ON ' . $db->quoteName('mp.virtuemart_product_id') . ' = ' . $db->quoteName('p.virtuemart_product_id'));
				
				if (count($manufacturers) == 1)
				{
					$query->where($db->quoteName('mp.virtuemart_manufacturer_id') . ' = ' . (int) $manufacturers[0]);
				}
				else
				{
					$query->where($db->quoteName('mp.virtuemart_manufacturer_id') . ' IN (' . implode(',', $manufacturers) . ')');
				}
			}
		}
		
		// Filter by parent product
		$parent_product_id = (int) $this->getState('filter.parent_product_id');
		
		if ($parent_product_id > 0)
		{
			$query->where($db->quoteName('p.product_parent_id') . ' = ' . $parent_product_id);
		}

		// Add the list ordering clause.
		$orderCol  = $this->state->get('list.ordering');
		$orderDirn = $this->state->get('list.direction');
		
		if ($orderCol && $orderDirn) 
		{
			$query->order($db->escape($orderCol . ' ' . $orderDirn));
		}

		// Add the list ordering clause.
		return $query;
	}

	public function getItems() 
	{
		if($items = parent::getItems())
		{
			// Do processing if required
		}
		
		return $items;
	}
	
	public function getCSV()
	{
		$db    = $this->getDbo();
		$query = $this->getListQuery();
		
		$db->setQuery($query);
		
		$items = $db->loadAssocList();
		
		if (empty($items))
		{
			$this->setError(JText::_('COM_VMREPORTER_NO_DATA_FOUND_INVENTORY'));
			
			return false;
		}
		
		$fields   = array_keys($items[0]);
		$headings = [];
		
		foreach ($fields as $field)
		{
			$label = VMReporterHelper::getFieldLabel($field);
			
			$headings[$field] = JText::_($label);
			
			if ($field === 'parent_product_name')
			{
				$headings['categories']    = JText::_('COM_VIRTUEMART_CATEGORY');
				$headings['manufacturers'] = JText::_('COM_VIRTUEMART_MANUFACTURER_S');
			}
		}
		
		JHtml::addIncludePath(JPATH_COMPONENT . '/helpers/html');
		JPluginHelper::importPlugin('vmreporter');
		
		if (version_compare(JVERSION, '4.0.0', 'ge'))
		{
			JFactory::getApplication()->triggerEvent('onPrepareCsvInventory', array(&$items, $headings));
		}
		else
		{
			JDispatcher::getInstance()->trigger('onPrepareCsvInventory', array(&$items, $headings));
		}
		
		ob_start();
		
		$df = fopen('php://output', 'w');
		
		// Set header
		fputcsv($df, array_values($headings));
		
		$order_ids = array();
		
		foreach($items as $row)
		{
			$row['categories']    = JHtml::_('vmreporter.categories', $row['virtuemart_product_id'], false);
			$row['manufacturers'] = JHtml::_('vmreporter.manufacturers', $row['virtuemart_product_id'], false);
			
			$row = array_intersect_key($row, $headings);
			$row = $this->sortColumns($row, array_keys($headings));
			
			fputcsv($df, $row);
		}
		
		fclose($df);
		
		return ob_get_clean();
	}
	
	public function sortColumns($report, $desired_order)
	{
		$keys = array_keys($report);
		$desired_order = array_intersect($desired_order, $keys);
		
		return array_merge(array_flip($desired_order), $report);
	}
}
