<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

jimport('joomla.filesystem.file');

/**
 * Reporter for VirtueMart
 */
class VMReporterModelReports extends JModelList
{
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'file_id', 'a.file_id',
				'created_time', 'a.created_time',
				'created_by', 'a.created_by',
				'name', 'u.name',
			);
		}
		
		parent::__construct($config);
	}

	/**
	* Method to auto-populate the model state.
	*
	* Note. Calling getState in this method will result in recursion.
	*/
	protected function populateState($ordering = null, $direction = null) 
	{
		// Initialise variables.
		$app = JFactory::getApplication();

		// Load the filter state.
		$search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
		$this->setState('filter.search', $search);
		
		// Load the parameters.
		$params = JComponentHelper::getParams('com_vmreporter');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.id', 'desc');
	}


	/**
	* Method to get a store id based on model configuration state.
	*
	* This is necessary because the model is used by the component and
	* different modules that might need different sets of data or different
	* ordering requirements.
	*
	* @param  string   $id A prefix for the store id.
	* @return string       A store id.
	* @since  1.0.0
	*/
	protected function getStoreId($id = '') 
	{
		// Compile the store id.
		$id.= ':' . $this->getState('filter.search');
		$id.= ':' . $this->getState('filter.state');

		return parent::getStoreId($id);
	}

	/**
	* Build an SQL query to load the list data.
	*
	* @return	JDatabaseQuery
	* @since	1.0.0
	*/
	protected function getListQuery() 
	{
		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState(
			'list.select', 'a.id, a.start_date, a.end_date, a.file_id, a.template_id, a.created_time, a.created_by, a.users_read'
			)
		);
		
		$query->from($db->quoteName('#__vmreporter_reports') . ' AS a');
		
		$query->select('u.name AS author')
			->join('LEFT', $db->quoteName('#__users') . ' AS u ON u.id = a.created_by');


		$query->select('t.name AS template_name')
			->join('INNER', $db->quoteName('#__vmreporter_templates') . ' AS t ON t.id = a.template_id');

		// Filter by search in title
		$search = $this->getState('filter.search');
		
		// Filter the items over the search string if set.
		if ($this->getState('filter.search') !== '')
		{
/*			// Escape the search token.
			$token = $db->quote('%' . $db->escape($this->getState('filter.search')) . '%');

			// Compile the different search clauses.
			$searches = array();
			$searches[] = 'a.country_name LIKE ' . $token;
			$searches[] = 'a.country_3_code LIKE ' . $token;
			$searches[] = 'a.country_2_code LIKE ' . $token;

			// Add the clauses to the query.
			$query->where('(' . implode(' OR ', $searches) . ')');*/
		}
	
		// Add the list ordering clause.
		$orderCol = $this->state->get('list.ordering');
		$orderDirn = $this->state->get('list.direction');
		
		if ($orderCol && $orderDirn) 
		{
			$query->order($db->escape($orderCol . ' ' . $orderDirn));
		}
		
		// Add the list ordering clause.
		return $query;
	}

	public function getItems() 
	{
		if($items = parent::getItems())
		{
			$params    = $this->state->get('params');
			$data_dir  = JPATH_SITE . '/media/vmreporter/data';
			$data_dir  = $params->get('data_dir', $data_dir);
			
			foreach($items as &$item)
			{
				$item->valid = true;
				
				if(!empty($item->file_id))
				{
					$data_file = JPath::clean($data_dir . '/' . $item->file_id . '.php');
					
					if(!JFile::exists($data_file))
					{
						$item->valid = false;
					}
				}
				
				$registry = new JRegistry($item->users_read);
				$item->users_read = $registry->toArray();
			}
		}
		
		return $items;
	}
}
