<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;

class VMReporterModelTemplate extends JModelAdmin
{
	/**
	 * @var  string  The prefix to use with controller messages.
	 */
	protected $text_prefix = 'COM_VMREPORTER';

	/**
	 * @var  string  Alias to manage history control
	 */
	public $typeAlias = 'com_vmreporter.template';

	/**
	 * @var null  Item data
	 */
	protected $item = null;
	
	protected $jsonProperties;
	
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JModelLegacy
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);
		
		$this->jsonProperties = array(
			'products', 'categories',
			'manufacturers', 'users',
			'shopper_groups', 'countries',
			'st_countries', 'statuses',
			'order_fields', 'order_item_fields',
			'bt_fields', 'st_fields', 
      'calculation_rules'
		);
	}
	
	/**
	 * Returns a reference to the a Table object, always creating it.
	 *
	 * @param   string  $type    The table type to instantiate
	 * @param   string  $prefix  A prefix for the table class name. Optional.
	 * @param   array   $config  Configuration array for model. Optional.
	 *
	 * @return    JTable    A database object
	 */
	public function getTable($type = 'Template', $prefix = 'VMReporterTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	/**
	 * Method to get the record form.
	 *
	 * @param   array    $data      An optional array of data for the form to interogate.
	 * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
	 *
	 * @return  JForm  A JForm object on success, false on failure
	 */
	public function getForm($data = array(), $loadData = true)
	{
		// Initialise variables.
		$app = JFactory::getApplication();

		// Get the form.
		$form = $this->loadForm('com_vmreporter.template', 'template', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}

		return $form;
	}

	/**
	 * Method to allow derived classes to preprocess the form.
	 *
	 * @param   JForm   $form   A JForm object.
	 * @param   mixed   $data   The data expected for the form.
	 * @param   string  $group  The name of the plugin group to import (defaults to "content").
	 *
	 * @return  void
	 *
	 * @see     JFormField
	 * @throws  Exception if there is an error in the form event.
	 */
	protected function preprocessForm(JForm $form, $data, $group = 'content')
	{
		if(!empty($form))
		{
			JPluginHelper::importPlugin('vmreporter');
			
			$newForm = new SimpleXMLElement('<form />');
			
			// Prepare order table field
			$orderFields = VMReporterHelper::getColumns('#__virtuemart_orders', array('order_billTax', 'user_currency_id', 'o_hash', 'locked_on', 'locked_by'), false);
			$extraFields = array('order_status_name', 'username', 'payment_name', 'shipment_name', 'user_currency');
			$orderFields = array_merge($orderFields, $extraFields);
			$orderDefaultFields = array('virtuemart_order_id', 'virtuemart_user_id', 'order_number', 'created_on', 'modified_on', 'order_total', 'order_billTaxAmount', 'order_billDiscountAmount', 'customer_number', 'payment_name', 'shipment_name', 'order_status', 'order_currency', 'username', 'order_status_name', 'coupon_discount', 'coupon_code', 'order_shipment', 'order_payment');
			
			$this->appendFormFields($newForm, 'order', $orderFields, $orderDefaultFields);
			
			// Prepare order item table fields
			$orderItemFields = VMReporterHelper::getColumns('#__virtuemart_order_items',  array('created_on', 'created_by', 'virtuemart_order_id', 'order_status', 'oi_hash', 'modified_on', 'modified_by', 'locked_on', 'locked_by'), false);
			$extraFields = array('item_order_status_name', 'item_order_status', 'category_id', 'category_name', 'manufacturer_id', 'manufacturer_name');
			$orderItemFields = array_merge($orderItemFields, $extraFields);
			$orderItemDefaultFields = array('virtuemart_product_id', 'order_item_sku', 'order_item_name', 'product_attribute', 'product_quantity', 'product_tax', 'product_final_price', 'product_subtotal_discount', 'product_subtotal_with_tax');
			
			$this->appendFormFields($newForm, 'order_item', $orderItemFields, $orderItemDefaultFields);
			
			// Prepare billing userinfo table fields
			$btFields = VMReporterHelper::getOrderUserinfoColumns('BT', null, false);
			
			if(in_array('virtuemart_country_id', $btFields))
			{
				$extraFields = array('country_3_code', 'country_name');
				$btFields = array_merge($btFields, $extraFields);
			}
			
			if(in_array('virtuemart_state_id', $btFields))
			{
				$extraFields = array('state_name');
				$btFields = array_merge($btFields, $extraFields);
			}
			
			$btDefaultFields = array('company', 'last_name', 'first_name', 'middle_name', 'phone', 'address_1', 'address_2', 'city', 'virtuemart_state_id', 'virtuemart_country_id', 'zip', 'email', 'customer_note');
			
			$this->appendFormFields($newForm, 'bt', $btFields, $btDefaultFields);
			
			// Prepare shipping userinfo table fields
			$stFields = VMReporterHelper::getOrderUserinfoColumns('ST', null, false);
			
			if(in_array('virtuemart_country_id', $stFields))
			{
				$extraFields = array('country_3_code', 'country_name');
				$stFields = array_merge($stFields, $extraFields);
			}
			
			if(in_array('virtuemart_state_id', $stFields))
			{
				$extraFields = array('state_name');
				$stFields = array_merge($stFields, $extraFields);
			}
			
			$stDefaultFields = array('company', 'last_name', 'first_name', 'middle_name', 'phone', 'address_1', 'address_2', 'city', 'virtuemart_state_id', 'virtuemart_country_id', 'zip');
			
			$this->appendFormFields($newForm, 'st', $stFields, $stDefaultFields);
			
			$form->load($newForm, false);
		}
		
		parent::preprocessForm($form, $data, $group);
	}
	
	protected function appendFormFields(&$form, $group, $columns, $defaults = array())
	{
		if(!empty($columns))
		{
			$fields = $form->addChild('fields');
			$fields->addAttribute('name', $group . '_fields');
			
			$fieldset = $fields->addChild('fieldset');
			$fieldset->addAttribute('name', $group);
			$fieldset->addAttribute('label', 'COM_VMREPORTER_FIELDSET_' . strtoupper($group));
			
			foreach($columns as $column)
			{
				$field = $fieldset->addChild('field');
				
				$field->addAttribute('name', $column);
				$field->addAttribute('type', 'checkbox');
				$field->addAttribute('label', VMReporterHelper::getFieldLabel($column));
				
				if(!empty($defaults) && in_array($column, $defaults))
				{
					$field->addAttribute('default', '1');
				}
			}
		}
	}

	/**
	 * Method to get the data that should be injected in the form.
	 *
	 * @return   mixed  The data for the form.
	 */
	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_vmreporter.edit.template.data', array());

		if (empty($data))
		{
			if ($this->item === null)
			{
				$this->item = $this->getItem();
			}

			$data = $this->item;
		}

		return $data;
	}

	/**
	 * Method to get a single record.
	 *
	 * @param   integer  $pk  The id of the primary key.
	 *
	 * @return  mixed    Object on success, false on failure.
	 */
	public function getItem($pk = null)
	{
		if($item = parent::getItem($pk))
		{
			foreach($this->jsonProperties as $property)
			{
				if(property_exists($item, $property) && is_string($item->$property))
				{
					$registry = new Registry($item->$property);

					$item->$property = $registry->toArray();
				}
			}
		}

		return $item;
	}

	/**
	 * Method to save the form data.
	 *
	 * @param   array  $data  The form data.
	 *
	 * @return  boolean  True on success, False on error.
	 */
	public function save($data)
	{
		$field_properties = array('order_fields', 'order_item_fields', 'bt_fields', 'st_fields');
		$field_selected = false;
		
		foreach($field_properties as $field_property)
		{
			if(!empty($data[$field_property]) && is_array($data[$field_property]))
			{
				foreach($data[$field_property] as $key => $value)
				{
					if($value > 0)
					{
						$field_selected = true;
						break;
					}
				}
			}
		}
		
		if(!$field_selected)
		{
			$this->setError(JText::_('COM_VMREPORTER_ERROR_NO_FIELDS_SELECTED'));
			return false;
		}
		
		foreach($this->jsonProperties as $property)
		{
			if (isset($data[$property]) && is_array($data[$property]))
			{
				$registry = new Registry($data[$property]);

				$data[$property] = $registry->toString();
			}
			else
			{
				$data[$property] = '';
			}
		}

		return parent::save($data);
	}

	/**
	 * Method to duplicate an Report
	 *
	 * @param   array  &$pks  An array of primary key IDs.
	 *
	 * @return  boolean  True if successful.
	 *
	 * @throws  Exception
	 */
	public function duplicate(&$pks)
	{
		$user = JFactory::getUser();

		// Access checks.
		if (!$user->authorise('core.create', 'com_vmreporter'))
		{
			throw new Exception(JText::_('JERROR_CORE_CREATE_NOT_PERMITTED'));
		}

		$dispatcher = JEventDispatcher::getInstance();
		$context    = $this->option . '.' . $this->name;

		// Include the plugins for the save events.
		JPluginHelper::importPlugin($this->events_map['save']);

		$table = $this->getTable();

		foreach ($pks as $pk)
		{
			if ($table->load($pk, true))
			{
				// Reset the id to create a new record.
				$table->id = 0;

				if (!$table->check())
				{
					throw new Exception($table->getError());
				}
				

				// Trigger the before save event.
				$result = $dispatcher->trigger($this->event_before_save, array($context, &$table, true));

				if (in_array(false, $result, true) || !$table->store())
				{
					throw new Exception($table->getError());
				}

				// Trigger the after save event.
				$dispatcher->trigger($this->event_after_save, array($context, &$table, true));
			}
			else
			{
				throw new Exception($table->getError());
			}
		}

		// Clean cache
		$this->cleanCache();

		return true;
	}
}