<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;
use Joomla\String\StringHelper;
use Joomla\Utilities\ArrayHelper;

/**
 * Cron Table class
 */
class VMReporterTableCron extends JTable 
{
	/**
	* Constructor
	*
	* @param JDatabase A database connector object
	*/
	public function __construct(&$db) 
	{
		parent::__construct('#__vmreporter_crons', 'id', $db);
		
		$this->setColumnAlias('published', 'state');
	}

	/**
	* Overloaded bind function to pre-process the params.
	*
	* @param  array           Named array
	* 
	* @return null|string     null is operation was satisfactory, otherwise returns an error
	*/
	public function bind($array, $ignore = '') 
	{
		return parent::bind($array, $ignore);
	}

	/**
	* Overloaded check function
	*/
	public function check() 
	{
		if (empty($this->time_interval))
		{
			$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_TIME_INERVAL'));
			return false;
		}
		
		if (empty($this->time_interval_uom))
		{
			$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_TIME_INTERVAL_UOM'));
			return false;
		}
		
		$db    = $this->getDbo();
		$query = $db->getQuery(true);
		$query->select($db->quoteName('id'))
			->from($this->_tbl)
			->where($db->quoteName('name') . ' = ' . $db->quote($this->name));
		
		if ($this->id > 0)
		{
			$query->where($db->quoteName('id') . ' <> ' . $db->quote($this->id));
		}
		
		$db->setQuery($query);
		$results = $db->loadColumn();
		
		if (!empty($results))
		{
			$input = JFactory::getApplication()->input;
			$task = $input->getString('task', '');
			
			if ($task == 'save2copy' || $task == 'duplicate')
			{
				$this->name = StringHelper::increment($this->name);
			}
			else
			{
				$this->setError(JText::_('COM_VMREPORTER_ERROR_CRON_NAME_EXISTS'));
				return false;
			}
		}
		
		if ($this->send_mail)
		{
			if (empty($this->email_ids))
			{
				$this->setError(JText::_('COM_VMREPORTER_ERROR_EMAIL_IDS_REQUIRED'));
				return false;
			}
			else
			{
				$email_ids = explode(',', $this->email_ids);
				
				foreach($email_ids as $key => &$email_id)
				{
					if (!JMailHelper::isEmailAddress($email_id))
					{
						unset($email_ids[$key]);
					}
				}
				
				if (empty($email_ids))
				{
					$this->setError(JText::_('COM_VMREPORTER_ERROR_EMAIL_IDS_INVALID'));
					return false;
				}
				
				$this->email_ids = implode(',', $email_ids);
			}
		}
		
		return parent::check();
	}

	/**
	* Method to store a node in the database table.
	*
	* @param   boolean  $updateNulls  True to update null values as well.
	*
	* @return  boolean  True on success.
	*/
	public function store($updateNulls = false)
	{
		$date = JFactory::getDate();
		$user = JFactory::getUser();
		$app  = JFactory::getApplication();
		$db   = $this->getDbo();

		if (empty($this->id))
		{
			$this->created_on = $date->toSql();
			$this->created_by = $user->get('id');
		}
		elseif ($app->isClient('administrator'))
		{
			$this->modified_on = $date->toSql();
			$this->modified_by = $user->get('id');
		}
		
		if (empty($this->start_time))
		{
			$this->start_time = $this->created_on;
		}
		else
		{
			$this->start_time = JFactory::getDate($this->start_time)->toSql();
		}
		
		$last_run_time = $this->start_time;
		
		if ((int) $this->last_run_time > 0)
		{
			$this->last_run_time = JFactory::getDate($this->last_run_time)->toSql();
		}
		else
		{
			$this->last_run_time = $db->getNullDate();
		}

		if ((int) $this->last_run_time > 0)
		{
			$last_run_time = $this->last_run_time;
		}
		
		$last_run_date = JFactory::getDate($last_run_time);
		
		if ($this->currently_running && $this->last_run_time > 0)
		{
			$since_last_run = $date->diff($last_run_date);
			
			$minutes  = $since_last_run->days * 24 * 60;
			$minutes += $since_last_run->h * 60;
			$minutes += $since_last_run->i;
			
			// If more than 15 minutes reset currently_running flag
			if ($minutes > 15)
			{
				$this->currently_running = 0;
			}
		}
		
		if ($this->time_interval_uom == 'months')
		{
			if ($this->time_interval > (29 * 86400))
			{
				$months = (int) ($this->time_interval / (30 * 86400));
			}
			else
			{
				$months = 1;
			}

			$last_run_date->add($this->add_months($months, $last_run_date));
		}
		else
		{
			$last_run_date->modify('+' . $this->time_interval . ' seconds');
		}
		
		$this->next_run_time = $last_run_date->toSql();
		
		if (!empty($this->log) && is_array($this->log))
		{
			$registry  = new Registry($this->log);
			$this->log = $registry->toString();
		}
		else
		{
			$this->log = '';
		}
		
		return parent::store($updateNulls);
	}
	
	protected function add_months($months, $date)
	{
		$next = JFactory::getDate($date->format('d-m-Y H:i:s'));
		
		$next->modify('last day of +' . $months . ' month');

		if ($date->format('d') > $next->format('d'))
		{
			return $date->diff($next);
		}
		else
		{
			return new DateInterval('P' . $months . 'M');
		}
	}

	public function setRunning($pk = null)
	{
		if (is_null($pk))
		{
			$pk = array();

			foreach ($this->_tbl_keys as $key)
			{
				$pk[$key] = $this->$key;
			}
		}
		elseif (!is_array($pk))
		{
			$pk = array($this->_tbl_key => $pk);
		}

		foreach ($this->_tbl_keys as $key)
		{
			$pk[$key] = is_null($pk[$key]) ? $this->$key : $pk[$key];

			if ($pk[$key] === null)
			{
				throw new UnexpectedValueException('Null primary key not allowed.');
			}
		}
		
		// Check the row in by primary key.
		$query = $this->_db->getQuery(true)
			->update($this->_tbl)
			->set($this->_db->quoteName('currently_running') . ' = 1');
		$this->appendPrimaryKeys($query, $pk);
		$this->_db->setQuery($query);
		$this->_db->execute();

		// Set table values in the object.
		$this->currently_running = 1;

		return true;
	}
	
	public function clearRunning($pk = null)
	{
		if (is_null($pk))
		{
			$pk = array();

			foreach ($this->_tbl_keys as $key)
			{
				$pk[$key] = $this->$key;
			}
		}
		elseif (!is_array($pk))
		{
			$pk = array($this->_tbl_key => $pk);
		}

		foreach ($this->_tbl_keys as $key)
		{
			$pk[$key] = is_null($pk[$key]) ? $this->$key : $pk[$key];

			if ($pk[$key] === null)
			{
				throw new UnexpectedValueException('Null primary key not allowed.');
			}
		}
		
		// Check the row in by primary key.
		$query = $this->_db->getQuery(true)
			->update($this->_tbl)
			->set($this->_db->quoteName('currently_running') . ' = 0');
		$this->appendPrimaryKeys($query, $pk);
		$this->_db->setQuery($query);
		$this->_db->execute();

		// Set table values in the object.
		$this->currently_running = 0;

		return true;
	}
}
