<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

use Joomla\Registry\Registry;
use Joomla\String\StringHelper;
use Joomla\Utilities\ArrayHelper;

/**
 * currency Table class
 */
class VMReporterTableTemplate extends JTable 
{
	/**
	* Constructor
	*
	* @param JDatabase A database connector object
	*/
	public function __construct(&$db) 
	{
		parent::__construct('#__vmreporter_templates', 'id', $db);
	}

	/**
	* Overloaded bind function to pre-process the params.
	*
	* @param   array           Named array
	* @return  null|string     null is operation was satisfactory, otherwise returns an error
	* 
	* @see     JTable:bind
	*/
	public function bind($array, $ignore = '') 
	{
		return parent::bind($array, $ignore);
	}

	/**
	* Overloaded check function
	*/
	public function check() 
	{
		if ($this->period == 'fixed')
		{
			$start_date = strtotime($this->start_date);
			$end_date   = strtotime($this->end_date);
			
			if (empty($start_date))
			{
				$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_START_DATE'));
				return false;
			}
			
			if (empty($end_date))
			{
				$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_END_DATE'));
				return false;
			}
			
			if ($end_date <= $start_date)
			{
				$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_DATE_INTERVAL'));
				return false;
			}
			
			$this->start_date = JFactory::getDate($this->start_date)->toSql();
			$this->end_date   = JFactory::getDate($this->end_date)->toSql();
		}
		else
		{
			$db = $this->getDbo();
			
			$this->start_date = $db->getNullDate();
			$this->end_date   = $db->getNullDate();
		}
		
		if ($this->period == 'custom_days')
		{
			$this->days = (int) $this->days;
			
			if (empty($this->days))
			{
				$this->setError(JText::_('COM_VMREPORTER_ERROR_INVALID_DAYS'));
				return false;
			}
		}
		
		$db    = $this->getDbo();
		$query = $db->getQuery(true);
    
		$query->select($db->quoteName('id'))
      ->from($this->_tbl)
      ->where($db->quoteName('name') . ' = ' . $db->quote($this->name));
		
		if ($this->id > 0)
		{
			$query->where($db->quoteName('id') . ' <> ' . $db->quote($this->id));
		}
		
		$db->setQuery($query);
		$results = $db->loadColumn();
		
		if (!empty($results))
		{
			$input = JFactory::getApplication()->input;
			$task  = $input->getString('task', '');
			
			if ($task == 'save2copy' || $task == 'duplicate')
			{
				$this->name = StringHelper::increment($this->name);
			}
			else
			{
				$this->setError(JText::_('COM_VMREPORTER_ERROR_TEMPLATE_NAME_EXISTS'));
				return false;
			}
		}
		
		if (empty($this->alias))
		{
			$this->alias = $this->name;
		}
		
		return parent::check();
	}

	/**
	* Method to store a node in the database table.
	*
	* @param   boolean  $updateNulls  True to update null values as well.
	*
	* @return  boolean  True on success.
	*/
	public function store($updateNulls = false)
	{
		$date = JFactory::getDate();
		$user = JFactory::getUser();

		if (empty($this->id))
		{
			$this->created_on = $date->toSql();
			$this->created_by = $user->get('id');
		}
		
		return parent::store($updateNulls);
	}
	
	/**
	 * Method to delete a row from the database table by primary key value.
	 *
	 * @param   mixed  $pk  An optional primary key value to delete.  If not set the instance property value is used.
	 *
	 * @return  boolean  True on success.
	 *
	 * @throws  UnexpectedValueException
	 */
	public function delete($pk = null)
	{
		if (is_null($pk))
		{
			$pk = array();

			foreach ($this->_tbl_keys as $key)
			{
				$pk[$key] = $this->$key;
			}
		}
		elseif (!is_array($pk))
		{
			$pk = array($this->_tbl_key => $pk);
		}

		foreach ($this->_tbl_keys as $key)
		{
			$pk[$key] = is_null($pk[$key]) ? $this->$key : $pk[$key];

			if ($pk[$key] === null)
			{
				throw new UnexpectedValueException('Null primary key not allowed.');
			}

			$this->$key = $pk[$key];
		}

		// Implement JObservableInterface: Pre-processing by observers
		$this->_observers->update('onBeforeDelete', array($pk));

		// If tracking assets, remove the asset first.
		if ($this->_trackAssets)
		{
			// Get the asset name
			$name  = $this->_getAssetName();
			/** @var JTableAsset $asset */
			$asset = self::getInstance('Asset');

			if ($asset->loadByName($name))
			{
				if (!$asset->delete())
				{
					$this->setError($asset->getError());

					return false;
				}
			}
		}
		
		$query = $this->_db->getQuery(true);
		
		// Check if reports or crons exist
		if (!empty($pk[$this->_tbl_key]))
		{
			$query->select('a.id, COUNT(DISTINCT r.id) AS report_count, COUNT(DISTINCT cr.id) AS cron_count')
	      ->from($this->_tbl . ' AS a')
	      ->join('LEFT', '#__vmreporter_reports AS r ON r.template_id = a.id')
	      ->join('LEFT', '#__vmreporter_crons AS cr ON cr.template_id = a.id')
	      ->group('a.id')
	      ->where('a.id = ' . (int) $pk[$this->_tbl_key]);

			$this->_db->setQuery($query);
			
			// Get counts
			$result = $this->_db->loadObject();
			
			if (!empty($result))
			{
				if (!empty($result->report_count) && !empty($result->cron_count))
				{
					$this->setError(JText::sprintf('COM_VMREPORTER_ERROR_TEMPLATE_HAS_CRONS_REPORTS', $this->name, $result->cron_count, $result->report_count));
					return false;
				}
				elseif (!empty($result->report_count))
				{
					$this->setError(JText::sprintf('COM_VMREPORTER_ERROR_TEMPLATE_HAS_REPORTS', $this->name, $result->report_count));
					return false;
				}
				elseif (!empty($result->cron_count))
				{
					$this->setError(JText::sprintf('COM_VMREPORTER_ERROR_TEMPLATE_HAS_CRONS', $this->name, $result->cron_count));
					return false;
				}
			}
		}

		// Delete the row by primary key.
		$query->clear()->delete($this->_tbl);
		$this->appendPrimaryKeys($query, $pk);

		$this->_db->setQuery($query);

		// Check for a database error.
		$this->_db->execute();

		// Implement JObservableInterface: Post-processing by observers
		$this->_observers->update('onAfterDelete', array($pk));

		return true;
	}
}
