/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

var VMReporter = window.VMReporter || {};

(function($, VMReporter) {

	VMReporter.overviewChart = null;

	VMReporter.analysisChart = null;

	VMReporter.showOverviewChart = function() {
		var chart_overview_data = VMReporter._('CHART_OVERVIEW_DATA', []);

		if ($.type(chart_overview_data) !== 'object') {
			alert('Invalid overview chart data object.');
			return false;
		}

		$('#vm-reporter-chart').removeClass('show-breakups');

		if (!VMReporter.overviewChart) {
			VMReporter.overviewChart = $.plot("#vmreporter-overview-chart", [{
				data: chart_overview_data.values.data,
				label: chart_overview_data.values.label,
				lines: {
					show: true
				},
			}, {
				data: chart_overview_data.counts.data,
				label: chart_overview_data.counts.label,
				yaxis: 2,
				lines: {
					show: true
				},
			}], {
				xaxes: [{
					mode: "time",
					tickColor: "#ddd"
				}],
				yaxes: [{
					min: 0,
					minTickSize: 1,
					tickColor: "#ddd",
					tickFormatter: VMReporter.currency_format,
				}, {
					alignTicksWithAxis: 1,
					position: 'right',
					minTickSize: 1,
					tickColor: "#ddd"
				}],
				legend: {
					position: "nw"
				},
				grid: {
					hoverable: true,
					borderColor: "#ddd"
				},
				tooltip: true,
				tooltipOpts: {
					//content: "%s: %y.00 <br/>%x", // show percentages, rounding to 2 decimal places
					content: function(xlabel, xvalue, yvalue) {
						var date = new Date(xvalue);
						date.setTime(date.getTime() + date.getTimezoneOffset() * 60 * 1000);
						return $.plot.formatDate(date, '%b %d, %Y') + ': <b>%y.0</b> [%s]';
					},
					shifts: {
						x: 20,
						y: 20
					},
					defaultTheme: false
				}
			});
		}
	};

	VMReporter.showAnalysisChart = function(chartData, content) {
		content = !content ? 'order_totals' : content;

		if (!chartData.hasOwnProperty(content)) {
			alert('Invalid chart data content: ' + content);
			return false;
		}

		var $placeHolder = $('#vmreporter-analysis-chart');

		$placeHolder.unbind();

		if (content == 'order_totals' || content == 'order_counts' || content == 'item_order_counts') {
			
			chartData[content].sort(function(a, b) {
				return a.total < b.total ? 1 : -1;
			});
			
			VMReporter.analysisChart = $.plot("#vmreporter-analysis-chart", chartData[content], {
				xaxes: [{
					mode: "time",
					tickColor: "#ddd"
				}],
				yaxes: [{
					min: 0,
					minTickSize: 1,
					tickColor: "#ddd",
					tickFormatter: function(v) {
						if (content == 'order_totals') {
							v = VMReporter.currency_format(v);
						}
						return v;
					},
				}],
				legend: {
					container: $('#vmreporter-analysis-legends'),
					labelFormatter: function(label, series) {
						if(typeof series.url !== typeof undefined && series.url) {
							label = '<a href="' + series.url + '" target="_blank">' + label + '</a>';
						}
						
						if(typeof series.total !== typeof undefined && series.total) {
							var total = content == 'order_totals' ? VMReporter.currency_format(series.total) : series.total;
							
							label += ' (' + total + ')';
						}
						
						return label;
					}
				},
				grid: {
					hoverable: true
				},
				points: {
					show: false
				},
				tooltip: true,
				tooltipOpts: {
					content: "%y.00", // show percentages, rounding to 2 decimal places
					content: function(xlabel, xvalue, yvalue) {
						var date = new Date(xvalue);
						date.setTime(date.getTime() + date.getTimezoneOffset() * 60 * 1000);
						var value = content == 'order_totals' ? VMReporter.currency_format(yvalue) : yvalue;
						return '%s<br />' + $.plot.formatDate(date, '%b %d, %Y') + ': <b>' + value + '</b>';
					},
					shifts: {
						x: 20,
						y: 20
					},
					defaultTheme: false
				}
			});
		} else if (content == 'revenues') {
			
			chartData[content].sort(function(a, b) {
				return a.data < b.data ? 1 : -1;
			});
			
			VMReporter.analysisChart = $.plot("#vmreporter-analysis-chart", chartData[content], {
				series: {
					pie: {
						show: true
					}
				},
				legend: {
					container: $('#vmreporter-analysis-legends'),
					labelFormatter: function(label, series) {
						if(typeof series.url !== typeof undefined && series.url) {
							label = '<a href="' + series.url + '" target="_blank">' + label + '</a>';
						}
						console.log(series.data);
						if(typeof series.data !== typeof undefined && typeof series.data[0] !== typeof undefined && typeof series.data[0][1] !== typeof undefined) {
							label += ' (' + VMReporter.currency_format(series.data[0][1]) + ')';
						}
						
						return label;
					}
				},
				grid: {
					hoverable: true
				},
				tooltip: true,
				tooltipOpts: {
					// content: "%p.00%, %y.0", // show percentages, rounding to 2 decimal places
					content: function(xlabel, xvalue, yvalue) {
						return '%s<br />%p.00%, ' + VMReporter.currency_format(yvalue);
					},
					shifts: {
						x: 20,
						y: 20
					},
					defaultTheme: false
				}
			});
		}
	};
	
	VMReporter.loader = '.vmreporter-chart-content.chart-breakup .vmreporter-loader-overlay, .vmreporter-chart-content.chart-breakup .vmreporter-loader';
	
	VMReporter.loaderTimeout = null;
	
	VMReporter.showLoader = function() {
		if(VMReporter.loaderTimeout) clearTimeout(VMReporter.loaderTimeout);
		
		$(VMReporter.loader).addClass('vmreporter-fade');
		
		VMReporter.loaderTimeout = setTimeout(function() {
			$(VMReporter.loader).addClass('vmreporter-in');
		}, 50);
	};

	VMReporter.hideLoader = function() {
		if(VMReporter.loaderTimeout) clearTimeout(VMReporter.loaderTimeout);
		
		$(VMReporter.loader).removeClass('vmreporter-in');
		
		VMReporter.loaderTimeout = setTimeout(function() {
			$(VMReporter.loader).removeClass('vmreporter-fade');
		}, 300);
	};

	// Initialize Angular App
	var reporterApp = angular.module('reporterApp', ['ngTouch', 'angularUtils.directives.dirPagination']);

	// Set pagination template
	reporterApp.config(function(paginationTemplateProvider) {
		paginationTemplateProvider.setPath(VMReporter._('ROOT', '') + 'media/vmreporter/tmpl/dirPagination.tpl.html');
	});

	// List data controller
	reporterApp.controller('listdata', function($scope, $filter) {

		$scope.rawitems = angular.fromJson(VMReporter._('ITEMS', []));
		$scope.sortKey = VMReporter._('ORDERSONLY', 0) ? 'virtuemart_order_id' : 'created_on';
		$scope.items = [];
		$scope.hasHiddenColumns = false;
		$scope.hiddenColumns = [];

		$scope.displayPerPage = $scope.displayPerPage || 20;

		$scope.init = function() {
			$scope.update();
			$('#vmreporter-table').removeClass('hide');
		};

		$scope.sort = function(keyname) {
			$scope.sortKey = keyname; //set the sortKey to the param passed
			$scope.reverse = !$scope.reverse; //if true make it false and vice versa
		};

		$scope.update = function() {
			if ($scope.uniqueOnly == 1) {
				if (!$scope.uniqueOrders) {
					$scope.uniqueOrders = [];
					var list = [];
					$scope.rawitems.forEach(function(item) {
						if (list[item.virtuemart_order_id] === undefined) {
							$scope.uniqueOrders.push(item);
							list[item.virtuemart_order_id] = 1;
						}
					});
				}
				$scope.items = $scope.uniqueOrders;
			} else {
				$scope.items = $scope.rawitems;
			}
		};

		$scope.hide = function(columnName, indexNumber) {
			$scope.hasHiddenColumns = true;
			$scope.hiddenColumns.push(columnName);
		};

		$scope.unhide = function() {
			$scope.hasHiddenColumns = false;
			$scope.hiddenColumns = [];
		};

		$scope.getColumnClass = function(columnName, value, type) {
			var cssClass = '';

			if (type == 'td') {
				var numberRegex = /^[0-9.,-]+$/;

				if (numberRegex.test(value)) {
					cssClass += 'col-number right';
				}
			}

			if ($scope.hiddenColumns.indexOf(columnName) >= 0) {
				cssClass += ' hide-column';
			}

			return cssClass;
		};
	});

	// Chart controller
	reporterApp.controller('chart', function($scope, $http) {

		$scope.data = {};
		$ajaxPool = null;

		$scope.initChart = function() {
			setTimeout(function() {
				VMReporter.showOverviewChart();
			}, 10);
		};

		$scope.updateChart = function() {
			var type = $scope.chartType;
			var content = $scope.chartContent;

			if (type == 'overview') {
				VMReporter.showOverviewChart();
			} else {
				if (!$scope.data.hasOwnProperty(type)) {
					if ($ajaxPool && $ajaxPool.readystate != 4) {
						$ajaxPool.abort();
					}

					$ajaxPool = $.ajax({
						type: 'GET',
						dataType: 'JSON',
						url: VMReporter._('BASE', '') + 'index.php?option=com_vmreporter&view=report&format=json&task=report.getAnalysis',
						data: {
							id: VMReporter._('ID', 0),
							analysis_type: type
						},
						cache: false,
						async: true,
						beforeSend: function() {
							VMReporter.showLoader();
						},
						success: function(result) {
							if ($.type(result) !== 'object' || result.error) {
								alert('An error was generated while fetching chart data.');
								return false;
							}

							$scope.data[type] = result.data;

							VMReporter.showAnalysisChart($scope.data[type], content);
							
							VMReporter.hideLoader();
						},
						error: function() {
							VMReporter.hideLoader();
							alert('An error was generated while fetching chart data.');
						}
					});
				} else {
					VMReporter.showAnalysisChart($scope.data[type], content);
				}
			}
		};
	});

})(jQuery, VMReporter);
