<?php
/**
 * @package     Reporter for VirtueMart
 * 
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined('_JEXEC') or die;

// Look for the component
@include_once(JPATH_ADMINISTRATOR . '/components/com_vmreporter/include.php');


class PlgSystemVMReporter extends JPlugin
{
	/**
	* 
	* @var object Application object
	* 
	*/
	protected $app;
	
	/**
	* 
	* @var boolean If we need to run the cron job
	* 
	*/
	protected $_run_cron;
	
	/**
	* Standard Joomla plugin constructor
	* 
	* @param undefined $subject
	* @param undefined $config
	* 
	* @return void
	*/
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
		
		$this->_run_cron = false;
	}
	
	/**
	 * After dispatch.
	 *
	 * @return  void
	 */
	public function onAfterDispatch()
	{
		$component_enabled = JComponentHelper::getComponent('com_vmreporter', true)->enabled || JComponentHelper::getComponent('vmreporter', true)->enabled;
		
		if (!defined('JPATH_VMREPORTER_ADMIN') || !$component_enabled)
		{
			return;
		}

		$check_active_crons = $this->_isHTML();
		
		if ($this->isAdmin() && $check_active_crons)
		{
			$option = strtolower($this->app->input->getCmd('option', ''));
			$view   = strtolower($this->app->input->getCmd('view', ''));
			
			// Don't run cron job when on cron or template editing page
			if ($option == 'com_vmreporter' && ($view == 'cron' || $view == 'template'))
			{
				$check_active_crons = false;
			}
		}
		
		if ($check_active_crons)
		{
			$config = JComponentHelper::getParams('com_vmreporter');
			
			if ($config->get('cronjob', 1))
			{
				$date  = JFactory::getDate();
				$db    = JFactory::getDbo();
				$query = $db->getQuery(true);
				
				$query->select(array($db->quoteName('id'), $db->quoteName('type'), $db->quoteName('time')))
					->from($db->quoteName('#__vmreporter_datetime'))
					->where($db->quoteName('type') . ' = ' . $db->quote('last:cron'));
				
				$db->setQuery($query);
				$cronTime = $db->loadObject();
				
				$dataUpdated = false;
				
				if (empty($cronTime) || !is_object($cronTime))
				{
					$this->_run_cron = true;
					
					$cronTime = new JObject;
					$cronTime->id = 0;
					$cronTime->type = 'last:cron';
					$cronTime->time = $date->toSql();
					
					$dataUpdated = true;
				}
				else
				{
					$cronjob_interval = (int) $config->get('cronjob_interval', 15);
					$lastCronTime = JFactory::getDate($cronTime->time);

					$lastCronTime->modify('+' . $cronjob_interval . ' minutes');

					if ($date >= $lastCronTime)
					{
						$this->_run_cron = true;
						
						$cronTime->time = $date->toSql();
						
						$dataUpdated = true;
					}
				}
				
				if ($dataUpdated)
				{
					if ($cronTime->id == 0)
					{
						$db->insertObject('#__vmreporter_datetime', $cronTime, 'id');
					}
					else
					{
						$db->updateObject('#__vmreporter_datetime', $cronTime, array('id'), false);
					}
				}
			}
		}
	}

	/**
	 * After render.
	 *
	 * @return  void
	 */
	public function onAfterRender()
	{
		if (!defined('JPATH_VMREPORTER_ADMIN'))
		{
			return;
		}
		
		// If we need to run the cron job
		if ($this->_run_cron)
		{
			// Intiate some variables
			$doc      = JFactory::getDocument();
			$uri      = JUri::getInstance();
			$tab      = $doc->_getTab();
			$lineEnd  = $doc->_getLineEnd();
			
			// Build cron job url
			$url  = $uri->toString(array('scheme', 'user', 'pass', 'host', 'port'));
			$url .= JUri::root(true) . '/';
			$url .= 'index.php?option=com_vmreporter&task=cronjob&_' . time();
			
			// Build html to initiate cron job
			$cron_html  = $tab . $tab . '<!--Initiate VirtueMart Reporter\'s Cron Job-->' . $lineEnd;
			$cron_html .= $tab . $tab . '<img src="' . htmlspecialchars($url, ENT_COMPAT, 'UTF-8', false) . '" width="1" height="1" alt="cronjob" class="sr-only" />' . $lineEnd;
			
			// Get body buffer
			$buffer = $this->app->getBody();
			
			// Add cron html just before the body closing tag
			$buffer = str_replace('</body>', $cron_html . '</body>', $buffer);
			
			// Set body buffer
			$this->app->setBody($buffer);
		}
	}
	
	/**
	 * On plugin's ajax call.
	 *
	 * @return json output
	 */
	public function onAjaxVmreporter()
	{
		if (!defined('JPATH_VMREPORTER_ADMIN'))
		{
			return;
		}
		
		if ($this->isAdmin())
		{
			// Register dlk helper class
			JLoader::register('VPComDlkHelper', JPATH_VMREPORTER_ADMIN . '/models/fields/vpcomdlk/helper.php');
			
			// Get an instance of dlk helper
			$dlkHelper = VPComDlkHelper::getInstance();
			$method    = strtolower($this->app->input->getCmd('method', ''));
			
			if ($method === 'validatedlk')
			{
				// Return validation result
				return $dlkHelper->validate();
			}
			elseif ($method === 'revalidatedlk')
			{
				// Return validation result
				return $dlkHelper->revalidate();
			}
			elseif ($method === 'clear')
			{
				// Return validation result
				return $dlkHelper->clear();
			}
		}
	}

	/**
	 * Called on live update installation.
	 *
	 * @return  boolean
	 */
	public function onInstallerBeforePackageDownload(&$url, &$headers)
	{
		if (!defined('JPATH_VMREPORTER_ADMIN'))
		{
			return;
		}
		
		// Register dlk helper class
		JLoader::register('VPComDlkHelper', JPATH_VMREPORTER_ADMIN . '/models/fields/vpcomdlk/helper.php');
		
		return VPComDlkHelper::getInstance()->addDlk($url, $headers);
	}
	
	/**
	* If the document is HTML
	* 
	* @return  boolean  True if HTML document
	*/
	private function _isHTML()
	{
		$doc   = JFactory::getDocument();
		$input = $this->app->input;
		
		$isFeed = ($doc->getType() == 'feed' || $input->getWord('format') == 'feed' || $input->getWord('type') == 'rss' || $input->getWord('type') == 'atom');
		$isHTML = ($doc->getType() == 'html' && !$isFeed);
		
		return $isHTML;
	}
	
	/**
	* Check if client is admin
	* 
	* @return boolean
	*/
	private function isAdmin()
	{
		return version_compare(JVERSION, '3.7.0', 'ge') ? $this->app->isClient('administrator') : $this->app->isAdmin();
	}
}
