<?php
/**
 * @package     Reporter for VirtueMart
 * @subpackage  Reporter for VirtueMart - Sales Commission
 * @copyright   Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Abhishek Das <info@virtueplanet.com> - https://www.virtueplanet.com
 */

defined ('_JEXEC') or die;

use Joomla\Registry\Registry;

/**
* Reporter for VirtueMart - Sales Commission Plugin
*/
class plgVmreporterCommission extends JPlugin
{
	protected $app;
	
	private static $_templates = array();
	
	public function onPrepareSortableFields(&$sortableFields, $template_id)
	{
		if (!$this->isApplicable($template_id))
		{
			return;
		}
		
		if (isset($sortableFields['order_total']))
		{
			unset($sortableFields['reporter_sales_commission']);

			$keys = array_keys($sortableFields);
			$vals = array_values($sortableFields);

			$insertAfter = array_search('order_total', $keys) + 1;

			$keys2 = array_splice($keys, $insertAfter);
			$vals2 = array_splice($vals, $insertAfter);

			$keys[] = 'reporter_sales_commission';
			$vals[] = $this->params->get('field_name', 'Commission');

			$sortableFields = array_merge(array_combine($keys, $vals), array_combine($keys2, $vals2));
		}
		else
		{
			$sortableFields['reporter_sales_commission'] = $this->params->get('field_name', 'Commission');
		}
	}
	
	public function onPrepareReportItems(&$items, $template_id)
	{
		if (!$this->isApplicable($template_id))
		{
			return;
		}
		
		$commissions = (array) $this->params->get('commissions', array());
		$commissions = array_map('floatval', $commissions);
		
		foreach ($items as &$item)
		{
			if (isset($item['order_salesPrice']) && isset($item['user_shoppergroups']))
			{
				if (empty($item['user_shoppergroups']) && $this->params->get('auto_assign_default_grouops', 1))
				{
					$item['virtuemart_user_id'] = !empty($item['virtuemart_user_id']) ? 2 : 1;
				}
				
				if (strpos($item['user_shoppergroups'], ',') !== false)
				{
					$shoppergroups = explode(',', $item['user_shoppergroups']);
				}
				else
				{
					$shoppergroups = array($item['user_shoppergroups']);
				}
				
				$shoppergroups = array_map('intval', $shoppergroups);
				$shoppergroups = array_values(array_filter($shoppergroups));
				$group_id = 0;
				
				if (!empty($shoppergroups))
				{
					if (count($shoppergroups) > 1)
					{
						if (!$template = $this->getTemplate($template_id))
						{
							return false;
						}
						
						$common_groups = array_intersect($shoppergroups, $template->shopper_groups);
						$common_groups = array_values($common_groups);
						
						if (!empty($common_groups))
						{
							$group_id = $common_groups[0];
						}
					}
					else
					{
						$group_id = $shoppergroups[0];
					}
				}
				
				$commission = 0;
				
				if ($group_id)
				{
					$commission = isset($commissions[$group_id]) ? $commissions[$group_id] : 0;
				}
				
				$item['reporter_sales_commission'] = $commission ? ($item['order_salesPrice'] * $commission) / 100 : '0.00';
			}
			else
			{
				$item['reporter_sales_commission'] = '0.00';
			}
			
			$roundoff = (int) $this->params->get('roundoff', 2);

			$item['reporter_sales_commission'] = $roundoff >=0 ? number_format(round($item['reporter_sales_commission'], $roundoff), $roundoff) : $item['reporter_sales_commission'];
			$item['reporter_sales_commission_raw'] = $item['reporter_sales_commission'];
		}
	}
	
	/**
	* Add order total row to CSV output
	* 
	* @param  array  $items           Report items
	* @param  array  $sortableFields  List of displayable fields
	* 
	* @return void
	*/
	public function onPrepareCsvItems(&$items, $sortableFields, $template_id, &$data)
	{
		if (!$this->isApplicable($template_id))
		{
			return;
		}
		
		if (isset($sortableFields['reporter_sales_commission']) && isset($items[0]) && isset($items[0]['reporter_sales_commission']))
		{
			$blank_row = array_fill_keys(array_keys($items[0]), '');
			$items[]   = $blank_row;
			$total_row = $blank_row;

			reset($sortableFields);
			
			$first_key = key($sortableFields);
			
			reset($sortableFields);
			
			$total_row[$first_key] = $this->params->get('total_field_name', 'TOTAL COMMISSION');
			$total_row['reporter_sales_commission'] = 0;
			
			$order_ids = array();
			
			foreach ($items as $item)
			{
				if (empty($item['virtuemart_order_id']) || in_array($item['virtuemart_order_id'], $order_ids))
				{
					continue;
				}
				
				$order_ids[] = $item['virtuemart_order_id'];
				
				$total_row['reporter_sales_commission'] += $item['reporter_sales_commission_raw'];
			}
			
			$roundoff = (int) $this->params->get('roundoff', 2);
			
			$total_row['reporter_sales_commission'] = $roundoff >=0 ? number_format(round($total_row['reporter_sales_commission'], $roundoff), $roundoff) : $total_row['reporter_sales_commission'];
			
			if ($this->params->get('show_in_mail', 0))
			{
				$mailData = new stdClass;
				$mailData->label = $this->params->get('total_field_name', 'TOTAL COMMISSION');
				$mailData->value = $total_row['reporter_sales_commission'];
				$mailData->isCurrency = true;
				
				$data['total_reporter_sales_commission'] = $mailData;
			}
			
			$items[] = $total_row;
		}
	}
	
	private function getTemplate($template_id)
	{
		if (!isset(self::$_templates[$template_id]))
		{
			JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_vmreporter/tables');
			
			$table = JTable::getInstance('Template', 'VMReporterTable', array());
			
			if (!$table->load($template_id))
			{
				JError::raiseWarning(500, $table->getError());
				return false;
			}
			
			$registry   = new Registry($table->shopper_groups);
			$table->shopper_groups = $registry->toArray();
			$table->shopper_groups = array_map('intval', $table->shopper_groups);
			
			self::$_templates[$template_id] = clone $table;
		}
		
		return self::$_templates[$template_id];
	}
	
	private function isApplicable($template_id)
	{
		if (empty($template_id))
		{
			return false;
		}
		
		$template_id = (int) $template_id;
		$templates = (array) $this->params->get('templates', array());
		$templates = array_map('intval', $templates);
		
		return in_array($template_id, $templates);
	}
}
